/*
 * Copyright (c) 1999-2008 Mark D. Hill and David A. Wood
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met: redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer;
 * redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution;
 * neither the name of the copyright holders nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Routingunit_d.C
 *
 * Niket Agarwal, Princeton University
 *
 * */

#include "mem/ruby/network/garnet-fixed-pipeline/RoutingUnit_d.hh"
#include "mem/ruby/network/garnet-fixed-pipeline/Router_d.hh"
#include "mem/ruby/network/garnet-fixed-pipeline/InputUnit_d.hh"
#include "mem/ruby/slicc_interface/NetworkMessage.hh"

RoutingUnit_d::RoutingUnit_d(Router_d *router)
{
        m_router = router;
        m_routing_table.clear();
        m_weight_table.clear();
}

void RoutingUnit_d::addRoute(const NetDest& routing_table_entry)
{
        m_routing_table.insertAtBottom(routing_table_entry);
}

void RoutingUnit_d::addWeight(int link_weight)
{
        m_weight_table.insertAtBottom(link_weight);
}

void RoutingUnit_d::RC_stage(flit_d *t_flit, InputUnit_d *in_unit, int invc)
{
        int outport = routeCompute(t_flit);
        in_unit->updateRoute(invc, outport);
        t_flit->advance_stage(VA_);
        m_router->vcarb_req();
}

int RoutingUnit_d::routeCompute(flit_d *t_flit)
{
        MsgPtr msg_ptr = t_flit->get_msg_ptr();
        NetworkMessage* net_msg_ptr = NULL;
        net_msg_ptr = dynamic_cast<NetworkMessage*>(msg_ptr.ref());
        NetDest msg_destination = net_msg_ptr->getInternalDestination();

        int output_link = -1;
        int min_weight = INFINITE_;

        for(int link = 0; link < m_routing_table.size(); link++)
        {
                if (msg_destination.intersectionIsNotEmpty(m_routing_table[link]))
                {
                        if(m_weight_table[link] >= min_weight)
                                continue;
                        output_link = link;
                        min_weight = m_weight_table[link];
                }
        }
        if(output_link == -1)
        {
                ERROR_MSG("Fatal Error:: No Route exists from this Router.");
                exit(0);
        }
        return output_link;

}
